@@ -31,7 +31,10 @@ from .utils import (
     subpaths
 )
 
-Parameters = namedtuple('Parameters', ['fields', 'include', 'sort'])
+Parameters = namedtuple(
+    'Parameters',
+    ['fields', 'include', 'sort', 'offset', 'limit']
+)
 
 json_array = sa.cast(
     postgresql.array([], type_=JSON), postgresql.ARRAY(JSON)
@@ -77,10 +80,13 @@ class QueryBuilder(object):
         Base url to be used for building JSON API compatible links objects. By
         default this is `None` indicating that no link objects will be built.
     """
-    def __init__(self, model_mapping, base_url=None):
+    def __init__(self, model_mapping, base_url=None, type_formatters=None):
         self.validate_model_mapping(model_mapping)
         self.resource_registry = ResourceRegistry(model_mapping)
         self.base_url = base_url
+        self.type_formatters = (
+            {} if type_formatters is None else type_formatters
+        )
 
     def validate_model_mapping(self, model_mapping):
         for model in model_mapping.values():
@@ -241,6 +247,10 @@ class QueryBuilder(object):
             List of dot-separated relationship paths.
         :param sort:
             List of attributes to apply as an order by for the root model.
+        :param limit:
+            Applies an SQL LIMIT to the generated query.
+        :param limit:
+            Applies an SQL OFFSET to the generated query.
         :param links:
             A dictionary of links to apply as top level links in the built
             query. Keys representing json keys and values as valid urls or
@@ -340,6 +350,8 @@ class SelectExpression(Expression):
         fields=None,
         include=None,
         sort=None,
+        limit=None,
+        offset=None,
         links=None,
         multiple=True,
         ids_only=False,
@@ -352,7 +364,9 @@ class SelectExpression(Expression):
         params = Parameters(
             fields=fields,
             include=include,
-            sort=sort
+            sort=sort,
+            limit=limit,
+            offset=offset
         )
         from_args = self._get_from_args(
             params,
@@ -388,11 +402,18 @@ class SelectExpression(Expression):
         )
         from_args = [data_query.as_scalar().label('data')]
 
-        if params.include is not None:
+        if params.include:
+            selectable = get_selectable(self.from_obj).original
+            if params.sort is not None:
+                selectable = apply_sort(selectable, selectable, params.sort)
+            if params.limit is not None:
+                selectable = selectable.limit(params.limit)
+            if params.offset is not None:
+                selectable = selectable.offset(params.offset)
             include_expr = IncludeExpression(
                 self.query_builder,
                 self.model,
-                get_selectable(self.from_obj).alias()
+                selectable.alias()
             )
             included_query = include_expr.build_included(params)
             from_args.append(included_query.as_scalar().label('included'))
@@ -406,6 +427,16 @@ class SelectExpression(Expression):
         return from_args
 
 
+def apply_sort(from_obj, query, sort):
+    for param in sort:
+        query = query.order_by(
+            sa.desc(getattr(from_obj.c, param[1:]))
+            if param[0] == '-' else
+            getattr(from_obj.c, param)
+        )
+    return query
+
+
 class AttributesExpression(Expression):
     @property
     def all_fields(self):
@@ -443,12 +474,22 @@ class AttributesExpression(Expression):
     def adapt_attribute(self, attr_name):
         cols = get_attrs(self.from_obj)
         hybrids = get_hybrid_properties(self.model).keys()
-        if attr_name in hybrids:
-            return ClauseAdapter(self.from_obj).traverse(
+        if (
+            attr_name in hybrids or
+            attr_name in self.column_property_expressions
+        ):
+            column = ClauseAdapter(self.from_obj).traverse(
                 getattr(self.model, attr_name)
             )
         else:
-            return getattr(cols, attr_name)
+            column = getattr(cols, attr_name)
+        return self.format_column(column)
+
+    def format_column(self, column):
+        for type_, formatter in self.query_builder.type_formatters.items():
+            if isinstance(column.type, type_):
+                return formatter(column)
+        return column
 
     def is_relationship_field(self, field):
         return field in get_mapper(self.model).relationships.keys()
@@ -489,11 +530,26 @@ class AttributesExpression(Expression):
 
     def validate_fields(self, fields):
         descriptors = get_all_descriptors(self.from_obj)
+        hybrids = get_hybrid_properties(self.model)
+        expressions = self.column_property_expressions
+
         for field in fields:
-            if field in get_hybrid_properties(self.model):
+            if field in hybrids or field in expressions:
                 continue
             self.validate_field(field, descriptors)
 
+    @property
+    def column_property_expressions(self):
+        return dict([
+            (key, attr)
+            for key, attr
+            in get_mapper(self.model).attrs.items()
+            if (
+                isinstance(attr, sa.orm.ColumnProperty) and
+                not isinstance(attr.columns[0], sa.Column)
+            )
+        ])
+
     def get_model_fields(self, fields):
         model_key = self.query_builder.get_resource_type(self.model)
 
@@ -538,10 +594,17 @@ class RelationshipsExpression(Expression):
             relationship.key,
             alias,
             get_selectable(self.from_obj),
-            order_by=relationship.order_by
+            order_by=self.build_order_by(relationship, alias)
         ).alias('relationships')
         return query
 
+    def build_order_by(self, relationship, alias):
+        return (
+            relationship.order_by
+            if relationship.order_by is not False
+            else [alias.id]
+        )
+
     def build_relationship_data_array(self, relationship, alias):
         query = self.build_relationship_data(relationship, alias)
         return sa.select([
@@ -647,16 +710,11 @@ class DataExpression(Expression):
         expr = self.build_data_expr(params, ids_only=ids_only)
         query = sa.select([expr], from_obj=self.from_obj)
         if params.sort is not None:
-            query = self.apply_sort(query, params.sort)
-        return query
-
-    def apply_sort(self, query, sort):
-        for param in sort:
-            query = query.order_by(
-                sa.desc(getattr(self.from_obj.c, param[1:]))
-                if param[0] == '-' else
-                getattr(self.from_obj.c, param)
-            )
+            query = apply_sort(self.from_obj, query, params.sort)
+        if params.limit is not None:
+            query = query.limit(params.limit)
+        if params.offset is not None:
+            query = query.offset(params.offset)
         return query
 
     def build_data_array(self, params, ids_only=False):
